@@ -307,7 +307,7 @@ if Crypto is not None and pyasn1 is not None:
     from twisted.conch.ssh import channel, connection, factory, keys
     from twisted.conch.ssh import transport, userauth
 
-    class UtilityTestCase(unittest.TestCase):
+    class UtilityTests(unittest.TestCase):
         def testCounter(self):
             c = transport._Counter('\x00\x00', 2)
             for i in xrange(256 * 256):
@@ -317,14 +317,6 @@ if Crypto is not None and pyasn1 is not None:
                 self.assertEqual(c(), struct.pack('!H', (i + 1) % (2 ** 16)))
 
 
-    class ConchTestPublicKeyChecker(checkers.SSHPublicKeyDatabase):
-        def checkKey(self, credentials):
-            blob = keys.Key.fromString(publicDSA_openssh).blob()
-            if credentials.username == 'testuser' and credentials.blob == blob:
-                return True
-            return False
-
-
     class ConchTestPasswordChecker:
         credentialInterfaces = checkers.IUsernamePassword,
 
@@ -446,7 +438,7 @@ if Crypto is not None and pyasn1 is not None:
     class ConchTestClientAuth(userauth.SSHUserAuthClient):
 
         hasTriedNone = 0 # have we tried the 'none' auth yet?
-        canSucceedPublicKey = 0 # can we succed with this yet?
+        canSucceedPublicKey = 0 # can we succeed with this yet?
         canSucceedPassword = 0
 
         def ssh_USERAUTH_SUCCESS(self, packet):
@@ -525,8 +517,20 @@ if Crypto is not None and pyasn1 is not None:
             self.onClose.callback(None)
 
 
+    def conchTestPublicKeyChecker():
+        """
+        Produces a SSHPublicKeyChecker with an in-memory key mapping with
+        a single use: 'testuser'
 
-class SSHProtocolTestCase(unittest.TestCase):
+        @return: L{twisted.conch.checkers.SSHPublicKeyChecker}
+        """
+        conchTestPublicKeyDB = checkers.InMemorySSHKeyDB(
+            {'testuser': [keys.Key.fromString(publicDSA_openssh)]})
+        return checkers.SSHPublicKeyChecker(conchTestPublicKeyDB)
+
+
+
+class SSHProtocolTests(unittest.TestCase):
     """
     Tests for communication between L{SSHServerTransport} and
     L{SSHClientTransport}.
@@ -549,7 +553,7 @@ class SSHProtocolTestCase(unittest.TestCase):
         p = portal.Portal(self.realm)
         sshpc = ConchTestSSHChecker()
         sshpc.registerChecker(ConchTestPasswordChecker())
-        sshpc.registerChecker(ConchTestPublicKeyChecker())
+        sshpc.registerChecker(conchTestPublicKeyChecker())
         p.registerChecker(sshpc)
         fac = ConchTestServerFactory()
         fac.portal = p
@@ -819,7 +823,7 @@ class SSHProtocolTestCase(unittest.TestCase):
 
 
 
-class TestSSHFactory(unittest.TestCase):
+class SSHFactoryTests(unittest.TestCase):
 
     if not Crypto:
         skip = "can't run w/o PyCrypto"
@@ -873,7 +877,7 @@ class TestSSHFactory(unittest.TestCase):
 
 
 
-class MPTestCase(unittest.TestCase):
+class MPTests(unittest.TestCase):
     """
     Tests for L{common.getMP}.
 
@@ -939,7 +943,7 @@ class MPTestCase(unittest.TestCase):
 
 
 
-class PyMPTestCase(MPTestCase):
+class PyMPTests(MPTests):
     """
     Tests for the python implementation of L{common.getMP}.
     """
@@ -947,14 +951,14 @@ class PyMPTestCase(MPTestCase):
 
 
 
-class GMPYMPTestCase(MPTestCase):
+class GMPYMPTests(MPTests):
     """
     Tests for the gmpy implementation of L{common.getMP}.
     """
     getMP = staticmethod(common._fastgetMP)
 
 
-class BuiltinPowHackTestCase(unittest.TestCase):
+class BuiltinPowHackTests(unittest.TestCase):
     """
     Tests that the builtin pow method is still correct after
     L{twisted.conch.ssh.common} monkeypatches it to use gmpy.
@@ -991,5 +995,5 @@ class BuiltinPowHackTestCase(unittest.TestCase):
 try:
     import gmpy
 except ImportError:
-    GMPYMPTestCase.skip = "gmpy not available"
+    GMPYMPTests.skip = "gmpy not available"
     gmpy = None
