@@ -8,7 +8,7 @@ Tests for the implementation of the ssh-userauth service.
 Maintainer: Paul Swartz
 """
 
-from zope.interface import implements
+from zope.interface import implementer
 
 from twisted.cred.checkers import ICredentialsChecker
 from twisted.cred.credentials import IUsernamePassword, ISSHPrivateKey
@@ -19,12 +19,15 @@ from twisted.cred.portal import IRealm, Portal
 from twisted.conch.error import ConchError, ValidPublicKey
 from twisted.internet import defer, task
 from twisted.protocols import loopback
+from twisted.python.reflect import requireModule
 from twisted.trial import unittest
 
-try:
-    import Crypto.Cipher.DES3
-    import pyasn1
-except ImportError:
+if requireModule('Crypto.Cipher.DES3') and requireModule('pyasn1'):
+    from twisted.conch.ssh.common import NS
+    from twisted.conch.checkers import SSHProtocolChecker
+    from twisted.conch.ssh import keys, userauth, transport
+    from twisted.conch.test import keydata
+else:
     keys = None
 
 
@@ -39,11 +42,6 @@ except ImportError:
             """
             A stub class so that later class definitions won't die.
             """
-else:
-    from twisted.conch.ssh.common import NS
-    from twisted.conch.checkers import SSHProtocolChecker
-    from twisted.conch.ssh import keys, userauth, transport
-    from twisted.conch.test import keydata
 
 
 
@@ -193,6 +191,7 @@ class FakeTransport(transport.SSHTransportBase):
 
 
 
+@implementer(IRealm)
 class Realm(object):
     """
     A mock realm for testing L{userauth.SSHUserAuthServer}.
@@ -200,22 +199,19 @@ class Realm(object):
     This realm is not actually used in the course of testing, so it returns the
     simplest thing that could possibly work.
     """
-    implements(IRealm)
-
 
     def requestAvatar(self, avatarId, mind, *interfaces):
         return defer.succeed((interfaces[0], None, lambda: None))
 
 
 
+@implementer(ICredentialsChecker)
 class PasswordChecker(object):
     """
     A very simple username/password checker which authenticates anyone whose
     password matches their username and rejects all others.
     """
     credentialInterfaces = (IUsernamePassword,)
-    implements(ICredentialsChecker)
-
 
     def requestAvatarId(self, creds):
         if creds.username == creds.password:
@@ -224,15 +220,13 @@ class PasswordChecker(object):
 
 
 
+@implementer(ICredentialsChecker)
 class PrivateKeyChecker(object):
     """
     A very simple public key checker which authenticates anyone whose
     public/private keypair is the same keydata.public/privateRSA_openssh.
     """
     credentialInterfaces = (ISSHPrivateKey,)
-    implements(ICredentialsChecker)
-
-
 
     def requestAvatarId(self, creds):
         if creds.blob == keys.Key.fromString(keydata.publicRSA_openssh).blob():
@@ -246,14 +240,13 @@ class PrivateKeyChecker(object):
 
 
 
+@implementer(ICredentialsChecker)
 class PAMChecker(object):
     """
     A simple PAM checker which asks the user for a password, verifying them
     if the password is the same as their username.
     """
     credentialInterfaces = (IPluggableAuthenticationModules,)
-    implements(ICredentialsChecker)
-
 
     def requestAvatarId(self, creds):
         d = creds.pamConversion([('Name: ', 2), ("Password: ", 1)])
@@ -265,16 +258,16 @@ class PAMChecker(object):
 
 
 
+@implementer(ICredentialsChecker)
 class AnonymousChecker(object):
     """
     A simple checker which isn't supported by L{SSHUserAuthServer}.
     """
     credentialInterfaces = (IAnonymous,)
-    implements(ICredentialsChecker)
 
 
 
-class SSHUserAuthServerTestCase(unittest.TestCase):
+class SSHUserAuthServerTests(unittest.TestCase):
     """
     Tests for SSHUserAuthServer.
     """
@@ -360,7 +353,7 @@ class SSHUserAuthServerTestCase(unittest.TestCase):
 
     def test_successfulPrivateKeyAuthentication(self):
         """
-        Test that private key authentication completes sucessfully,
+        Test that private key authentication completes successfully,
         """
         blob = keys.Key.fromString(keydata.publicRSA_openssh).blob()
         obj = keys.Key.fromString(keydata.privateRSA_openssh)
@@ -717,7 +710,7 @@ class SSHUserAuthServerTestCase(unittest.TestCase):
 
 
 
-class SSHUserAuthClientTestCase(unittest.TestCase):
+class SSHUserAuthClientTests(unittest.TestCase):
     """
     Tests for SSHUserAuthClient.
     """
@@ -811,47 +804,6 @@ class SSHUserAuthClientTestCase(unittest.TestCase):
                  NS('none'))])
 
 
-    def test_old_publickey_getPublicKey(self):
-        """
-        Old SSHUserAuthClients returned strings of public key blobs from
-        getPublicKey().  Test that a Deprecation warning is raised but the key is
-        verified correctly.
-        """
-        oldAuth = OldClientAuth('foo', FakeTransport.Service())
-        oldAuth.transport = FakeTransport(None)
-        oldAuth.transport.sessionID = 'test'
-        oldAuth.serviceStarted()
-        oldAuth.transport.packets = []
-        self.assertWarns(DeprecationWarning, "Returning a string from "
-                         "SSHUserAuthClient.getPublicKey() is deprecated since "
-                         "Twisted 9.0.  Return a keys.Key() instead.",
-                         userauth.__file__, oldAuth.tryAuth, 'publickey')
-        self.assertEqual(oldAuth.transport.packets, [
-                (userauth.MSG_USERAUTH_REQUEST, NS('foo') + NS('nancy') +
-                 NS('publickey') + '\x00' + NS('ssh-rsa') +
-                 NS(keys.Key.fromString(keydata.publicRSA_openssh).blob()))])
-
-
-    def test_old_publickey_getPrivateKey(self):
-        """
-        Old SSHUserAuthClients returned a PyCrypto key object from
-        getPrivateKey().  Test that _cbSignData signs the data warns the
-        user about the deprecation, but signs the data correctly.
-        """
-        oldAuth = OldClientAuth('foo', FakeTransport.Service())
-        d = self.assertWarns(DeprecationWarning, "Returning a PyCrypto key "
-                             "object from SSHUserAuthClient.getPrivateKey() is "
-                             "deprecated since Twisted 9.0.  "
-                             "Return a keys.Key() instead.", userauth.__file__,
-                             oldAuth.signData, None, 'data')
-        def _checkSignedData(sig):
-            self.assertEqual(sig,
-                keys.Key.fromString(keydata.privateRSA_openssh).sign(
-                    'data'))
-        d.addCallback(_checkSignedData)
-        return d
-
-
     def test_no_publickey(self):
         """
         If there's no public key, auth_publickey should return a Deferred
@@ -994,7 +946,7 @@ class SSHUserAuthClientTestCase(unittest.TestCase):
 
 
 
-class LoopbackTestCase(unittest.TestCase):
+class LoopbackTests(unittest.TestCase):
 
 
     if keys is None:
@@ -1063,7 +1015,7 @@ class LoopbackTestCase(unittest.TestCase):
 
 
 
-class ModuleInitializationTestCase(unittest.TestCase):
+class ModuleInitializationTests(unittest.TestCase):
     if keys is None:
         skip = "cannot run w/o PyCrypto or PyASN1"
 
