@@ -11,6 +11,9 @@ from __future__ import division, absolute_import
 
 from zope.interface import Interface, Attribute
 
+from twisted.python import deprecate
+from twisted.python.versions import Version
+
 
 class IAddress(Interface):
     """
@@ -650,7 +653,9 @@ class IReactorTCP(Interface):
         """
         Connect a TCP client.
 
-        @param host: a host name
+        @param host: A hostname or an IPv4 or IPv6 address literal.
+
+        @type host: L{bytes}
 
         @param port: a port number
 
@@ -870,7 +875,7 @@ class IReactorMulticast(Interface):
     UDP socket methods that support multicast.
 
     IMPORTANT: This is an experimental new interface. It may change
-    without backwards compatability. Suggestions are welcome.
+    without backwards compatibility. Suggestions are welcome.
     """
 
     def listenMulticast(port, protocol, interface='', maxPacketSize=8192,
@@ -2128,6 +2133,62 @@ class IUNIXTransport(ITransport):
 
 
 
+class IOpenSSLServerConnectionCreator(Interface):
+    """
+    A provider of L{IOpenSSLServerConnectionCreator} can create
+    L{OpenSSL.SSL.Connection} objects for TLS servers.
+
+    @see: L{twisted.internet.ssl}
+
+    @note: Creating OpenSSL connection objects is subtle, error-prone, and
+        security-critical.  Before implementing this interface yourself,
+        consider using L{twisted.internet.ssl.CertificateOptions} as your
+        C{contextFactory}.  (For historical reasons, that class does not
+        actually I{implement} this interface; nevertheless it is usable in all
+        Twisted APIs which require a provider of this interface.)
+    """
+
+    def serverConnectionForTLS(tlsProtocol):
+        """
+        Create a connection for the given server protocol.
+
+        @param tlsProtocol: the protocol server making the request.
+        @type tlsProtocol: L{twisted.protocols.tls.TLSMemoryBIOProtocol}.
+
+        @return: an OpenSSL connection object configured appropriately for the
+            given Twisted protocol.
+        @rtype: L{OpenSSL.SSL.Connection}
+        """
+
+
+
+class IOpenSSLClientConnectionCreator(Interface):
+    """
+    A provider of L{IOpenSSLClientConnectionCreator} can create
+    L{OpenSSL.SSL.Connection} objects for TLS clients.
+
+    @see: L{twisted.internet.ssl}
+
+    @note: Creating OpenSSL connection objects is subtle, error-prone, and
+        security-critical.  Before implementing this interface yourself,
+        consider using L{twisted.internet.ssl.optionsForClientTLS} as your
+        C{contextFactory}.
+    """
+
+    def clientConnectionForTLS(tlsProtocol):
+        """
+        Create a connection for the given client protocol.
+
+        @param tlsProtocol: the client protocol making the request.
+        @type tlsProtocol: L{twisted.protocols.tls.TLSMemoryBIOProtocol}.
+
+        @return: an OpenSSL connection object configured appropriately for the
+            given Twisted protocol.
+        @rtype: L{OpenSSL.SSL.Connection}
+        """
+
+
+
 class ITLSTransport(ITCPTransport):
     """
     A TCP transport that supports switching to TLS midstream.
@@ -2139,9 +2200,21 @@ class ITLSTransport(ITCPTransport):
         """
         Initiate TLS negotiation.
 
-        @param contextFactory: A context factory (see L{ssl.py<twisted.internet.ssl>})
+        @param contextFactory: An object which creates appropriately configured
+            TLS connections.
+
+            For clients, use L{twisted.internet.ssl.optionsForClientTLS}; for
+            servers, use L{twisted.internet.ssl.CertificateOptions}.
+
+        @type contextFactory: L{IOpenSSLClientConnectionCreator} or
+            L{IOpenSSLServerConnectionCreator}, depending on whether this
+            L{ITLSTransport} is a server or not.  If the appropriate interface
+            is not provided by the value given for C{contextFactory}, it must
+            be an old-style L{twisted.internet.ssl.ContextFactory} or similar.
         """
 
+
+
 class ISSLTransport(ITCPTransport):
     """
     A SSL/TLS based transport.
@@ -2153,6 +2226,36 @@ class ISSLTransport(ITCPTransport):
         """
 
 
+
+class ICipher(Interface):
+    """
+    A TLS cipher.
+    """
+    fullName = Attribute(
+        "The fully qualified name of the cipher in L{unicode}."
+    )
+
+
+
+class IAcceptableCiphers(Interface):
+    """
+    A list of acceptable ciphers for a TLS context.
+    """
+    def selectCiphers(availableCiphers):
+        """
+        Choose which ciphers to allow to be negotiated on a TLS connection.
+
+        @param availableCiphers: A L{list} of L{ICipher} which gives the names
+            of all ciphers supported by the TLS implementation in use.
+
+        @return: A L{list} of L{ICipher} which represents the ciphers
+            which may be negotiated on the TLS connection.  The result is
+            ordered by preference with more preferred ciphers appearing
+            earlier.
+        """
+
+
+
 class IProcessTransport(ITransport):
     """
     A process transport.
@@ -2282,7 +2385,10 @@ class IUDPTransport(Interface):
 
     def getHost():
         """
-        Returns L{IPv4Address}.
+        Get this port's host address.
+
+        @return: an address describing the listening port.
+        @rtype: L{IPv4Address} or L{IPv6Address}.
         """
 
     def stopListening():
@@ -2293,6 +2399,21 @@ class IUDPTransport(Interface):
         upon completion.
         """
 
+    def setBroadcastAllowed(enabled):
+        """
+        Set whether this port may broadcast.
+
+        @param enabled: Whether the port may broadcast.
+        @type enabled: L{bool}
+        """
+
+    def getBroadcastAllowed():
+        """
+        Checks if broadcast is currently allowed on this port.
+
+        @return: Whether this port may broadcast.
+        @rtype: L{bool}
+        """
 
 
 class IUNIXDatagramTransport(Interface):
@@ -2398,7 +2519,8 @@ class IStreamClientEndpoint(Interface):
 
         @param protocolFactory: A provider of L{IProtocolFactory}
         @return: A L{Deferred} that results in an L{IProtocol} upon successful
-            connection otherwise a L{ConnectError}
+            connection otherwise a L{Failure} wrapping L{ConnectError} or
+            L{NoProtocol <twisted.internet.error.NoProtocol>}.
         """
 
 
@@ -2453,6 +2575,9 @@ class IStreamServerEndpointStringParser(Interface):
 
 class IStreamClientEndpointStringParser(Interface):
     """
+    This interface is deprecated since Twisted 14.0; please use the
+    L{IStreamClientEndpointStringParserWithReactor} interface instead.
+
     An L{IStreamClientEndpointStringParser} is a parser which can convert
     a set of string C{*args} and C{**kwargs} into an L{IStreamClientEndpoint}
     provider.
@@ -2465,6 +2590,11 @@ class IStreamClientEndpointStringParser(Interface):
     C{twisted.plugins} package, that plugin's C{parseStreamClient} method will
     be used to produce endpoints for any description string that begins with
     the result of that L{IStreamClientEndpointStringParser}'s prefix attribute.
+
+    If a L{IStreamClientEndpointStringParserWithReactor} plugin and
+    L{IStreamClientEndpointStringParser} plugin share the same prefix, the
+    L{IStreamClientEndpointStringParserWithReactor} plugin will be preferred.
+
     """
 
     prefix = Attribute(
@@ -2497,3 +2627,68 @@ class IStreamClientEndpointStringParser(Interface):
         @return: a client endpoint
         @rtype: L{IStreamClientEndpoint}
         """
+
+deprecate.deprecatedModuleAttribute(
+    Version("Twisted", 14, 0, 0),
+    "This interface has been superseded by "
+    "IStreamClientEndpointStringParserWithReactor.",
+    __name__,
+    "IStreamClientEndpointStringParser")
+
+
+
+class IStreamClientEndpointStringParserWithReactor(Interface):
+    """
+    An L{IStreamClientEndpointStringParserWithReactor} is a parser which can
+    convert a set of string C{*args} and C{**kwargs} into an
+    L{IStreamClientEndpoint} provider. It's much like
+    L{IStreamClientEndpointStringParser}, except that the reactor is passed
+    along to L{parseStreamClient} too.
+
+    This interface is really only useful in the context of the plugin system
+    for L{endpoints.clientFromString}.  See the document entitled "I{The
+    Twisted Plugin System}" for more details on how to write a plugin.
+
+    If you place an L{IStreamClientEndpointStringParserWithReactor} plugin in
+    the C{twisted.plugins} package, that plugin's C{parseStreamClient} method
+    will be used to produce endpoints for any description string that begins
+    with the result of that L{IStreamClientEndpointStringParserWithReactor}'s
+    prefix attribute.
+
+    If a L{IStreamClientEndpointStringParserWithReactor} plugin and
+    L{IStreamClientEndpointStringParser} plugin share the same prefix, the
+    L{IStreamClientEndpointStringParserWithReactor} plugin will be preferred.
+    """
+
+    prefix = Attribute(
+        """
+        L{bytes}, the description prefix to respond to.  For example, an
+        L{IStreamClientEndpointStringParserWithReactor} plugin which had
+        C{b"foo"} for its C{prefix} attribute would be called for endpoint
+        descriptions like C{b"foo:bar:baz"} or C{b"foo:"}.
+        """
+    )
+
+
+    def parseStreamClient(reactor, *args, **kwargs):
+        """
+        This method is invoked by L{endpoints.clientFromString}, if the type of
+        endpoint matches the return value from this
+        L{IStreamClientEndpointStringParserWithReactor}'s C{prefix} method.
+
+        @param reactor: The reactor passed to L{endpoints.clientFromString}.
+
+        @param args: The byte string arguments, minus the endpoint type, in the
+            endpoint description string, parsed according to the rules
+            described in L{endpoints.quoteStringArgument}.  For example, if the
+            description were C{b"my-type:foo:bar:baz=qux"}, C{args} would be
+            C{(b'foo', b'bar')}
+
+        @param kwargs: The byte string arguments from the endpoint description
+            passed as keyword arguments.  For example, if the description were
+            C{b"my-type:foo:bar:baz=qux"}, C{kwargs} would be
+            C{dict(baz=b'qux')}.
+
+        @return: a client endpoint
+        @rtype: a provider of L{IStreamClientEndpoint}
+        """
