@@ -34,7 +34,7 @@ else:
 
 
 
-class UtilTestCase(unittest.TestCase):
+class UtilTests(unittest.TestCase):
 
     def testUniq(self):
         l = ["a", 1, "ab", "a", 3, 4, 1, 2, 2, 4, 6]
@@ -147,7 +147,7 @@ class UntilConcludesTests(unittest.TestCase):
 
 
 
-class SwitchUIDTest(unittest.TestCase):
+class SwitchUIDTests(unittest.TestCase):
     """
     Tests for L{util.switchUID}.
     """
@@ -199,11 +199,12 @@ class SwitchUIDTest(unittest.TestCase):
         util.switchUID(uid, None)
         self.assertEqual(self.initgroupsCalls, [])
         self.assertEqual(self.mockos.actions, [])
-        warnings = self.flushWarnings([util.switchUID])
-        self.assertEqual(len(warnings), 1)
-        self.assertIn('tried to drop privileges and setuid %i' % uid, 
-                      warnings[0]['message'])
-        self.assertIn('but uid is already %i' % uid, warnings[0]['message'])
+        currentWarnings = self.flushWarnings([util.switchUID])
+        self.assertEqual(len(currentWarnings), 1)
+        self.assertIn('tried to drop privileges and setuid %i' % uid,
+                      currentWarnings[0]['message'])
+        self.assertIn(
+            'but uid is already %i' % uid, currentWarnings[0]['message'])
 
 
     def test_currentEUID(self):
@@ -215,15 +216,16 @@ class SwitchUIDTest(unittest.TestCase):
         util.switchUID(euid, None, True)
         self.assertEqual(self.initgroupsCalls, [])
         self.assertEqual(self.mockos.seteuidCalls, [])
-        warnings = self.flushWarnings([util.switchUID])
-        self.assertEqual(len(warnings), 1)
-        self.assertIn('tried to drop privileges and seteuid %i' % euid, 
-                      warnings[0]['message'])
-        self.assertIn('but euid is already %i' % euid, warnings[0]['message'])
+        currentWarnings = self.flushWarnings([util.switchUID])
+        self.assertEqual(len(currentWarnings), 1)
+        self.assertIn('tried to drop privileges and seteuid %i' % euid,
+                      currentWarnings[0]['message'])
+        self.assertIn(
+            'but euid is already %i' % euid, currentWarnings[0]['message'])
 
 
 
-class TestMergeFunctionMetadata(unittest.TestCase):
+class MergeFunctionMetadataTests(unittest.TestCase):
     """
     Tests for L{mergeFunctionMetadata}.
     """
@@ -320,7 +322,7 @@ class TestMergeFunctionMetadata(unittest.TestCase):
 
 
 
-class OrderedDictTest(unittest.TestCase):
+class OrderedDictTests(unittest.TestCase):
     def testOrderedDict(self):
         d = util.OrderedDict()
         d['a'] = 'b'
@@ -348,7 +350,7 @@ class OrderedDictTest(unittest.TestCase):
 
 
 
-class InsensitiveDictTest(unittest.TestCase):
+class InsensitiveDictTests(unittest.TestCase):
     """
     Tests for L{util.InsensitiveDict}.
     """
@@ -365,9 +367,13 @@ class InsensitiveDictTest(unittest.TestCase):
         self.assertEqual(dct['foo'], dct.get('Foo'))
         self.assertIn(1, dct)
         self.assertIn('foo', dct)
-        # Make eval() work, urrrrgh:
-        InsensitiveDict = util.InsensitiveDict
-        self.assertEqual(eval(repr(dct)), dct)
+
+        result = eval(repr(dct), {
+            'dct': dct,
+            'InsensitiveDict': util.InsensitiveDict,
+            })
+        self.assertEqual(result, dct)
+
         keys=['Foo', 'fnz', 1]
         for x in keys:
             self.assertIn(x, dct.keys())
@@ -431,7 +437,7 @@ class PasswordTestingProcessProtocol(ProcessProtocol):
         self.finished.callback((reason, self.output))
 
 
-class GetPasswordTest(unittest.TestCase):
+class GetPasswordTests(unittest.TestCase):
     if not IReactorProcess.providedBy(reactor):
         skip = "Process support required to test getPassword"
 
@@ -464,7 +470,7 @@ class GetPasswordTest(unittest.TestCase):
 
 
 
-class SearchUpwardsTest(unittest.TestCase):
+class SearchUpwardsTests(unittest.TestCase):
     def testSearchupwards(self):
         os.makedirs('searchupwards/a/b/c')
         file('searchupwards/foo.txt', 'w').close()
@@ -488,7 +494,7 @@ class SearchUpwardsTest(unittest.TestCase):
 
 
 
-class IntervalDifferentialTestCase(unittest.TestCase):
+class IntervalDifferentialTests(unittest.TestCase):
     def testDefault(self):
         d = iter(util.IntervalDifferential([], 10))
         for i in range(100):
@@ -656,7 +662,7 @@ class EqualityTests(unittest.TestCase):
 
     def test_unequality(self):
         """
-        Unequality between instances of a particular L{record} should be
+        Inequality between instances of a particular L{record} should be
         defined as the negation of equality.
         """
         self.assertFalse(Record(1, 2) != Record(1, 2))
@@ -928,14 +934,14 @@ class DeprecationTests(unittest.TestCase):
         L{util.getPluginDirs} is deprecated.
         """
         util.getPluginDirs()
-        warnings = self.flushWarnings(offendingFunctions=[
+        currentWarnings = self.flushWarnings(offendingFunctions=[
             self.test_getPluginDirs])
         self.assertEqual(
-            warnings[0]['message'],
+            currentWarnings[0]['message'],
             "twisted.python.util.getPluginDirs is deprecated since Twisted "
             "12.2.")
-        self.assertEqual(warnings[0]['category'], DeprecationWarning)
-        self.assertEqual(len(warnings), 1)
+        self.assertEqual(currentWarnings[0]['category'], DeprecationWarning)
+        self.assertEqual(len(currentWarnings), 1)
 
 
     def test_addPluginDir(self):
@@ -943,14 +949,14 @@ class DeprecationTests(unittest.TestCase):
         L{util.addPluginDir} is deprecated.
         """
         util.addPluginDir()
-        warnings = self.flushWarnings(offendingFunctions=[
+        currentWarnings = self.flushWarnings(offendingFunctions=[
             self.test_addPluginDir])
         self.assertEqual(
-            warnings[0]['message'],
+            currentWarnings[0]['message'],
             "twisted.python.util.addPluginDir is deprecated since Twisted "
             "12.2.")
-        self.assertEqual(warnings[0]['category'], DeprecationWarning)
-        self.assertEqual(len(warnings), 1)
+        self.assertEqual(currentWarnings[0]['category'], DeprecationWarning)
+        self.assertEqual(len(currentWarnings), 1)
     test_addPluginDir.suppress = [
             SUPPRESS(category=DeprecationWarning,
                      message="twisted.python.util.getPluginDirs is deprecated")
@@ -1077,9 +1083,80 @@ class FancyStrMixinTests(unittest.TestCase):
         obj = Foo()
         self.assertEqual(str(obj), repr(obj))
 
+
+
+class PadToTests(unittest.TestCase):
+    """
+    Tests for L{util.padTo}.
+    """
+
+    def test_default(self):
+        """
+        C{None} values can be added to a list to cause it to have a certain
+        length.
+        """
+        padded = util.padTo(3, [])
+        self.assertEqual([None] * 3, padded)
+
+
+    def test_specificDefaultValue(self):
+        """
+        A specific value can be added to a list to cause it to have a certain
+        length.
+        """
+        padded = util.padTo(4, [], "x")
+        self.assertEqual(["x"] * 4, padded)
+
+
+    def test_padNonEmptyList(self):
+        """
+        A list which already has some items has the padding value added after
+        those items.
+        """
+        padded = util.padTo(3, [1, 2], "z")
+        self.assertEqual([1, 2, "z"], padded)
+
+
+    def test_padToSmallerSize(self):
+        """
+        L{util.padTo} can't pad a list if the size requested is smaller than
+        the size of the list to pad.
+        """
+        self.assertRaises(ValueError, util.padTo, 1, [1, 2])
+
+
+    def test_alreadyPadded(self):
+        """
+        If the list is already the length indicated by the padding argument
+        then a list with the same value is returned.
+        """
+        items = [1, 2]
+        padded = util.padTo(len(items), items)
+        self.assertEqual(items, padded)
+
+
+    def test_alreadyPaddedCopies(self):
+        """
+        If the list is already the length indicated by the padding argument
+        then the return value is a copy of the input.
+        """
+        items = [1, 2]
+        padded = util.padTo(len(items), items)
+        self.assertIsNot(padded, items)
+
+
+    def test_makeCopy(self):
+        """
+        L{util.padTo} doesn't modify the input list but makes a copy.
+        """
+        items = []
+        util.padTo(4, items)
+        self.assertEqual([], items)
+
+
 if _PY3:
-    del (SwitchUIDTest, SearchUpwardsTest, RunAsEffectiveUserTests,
-         OrderedDictTest, IntervalDifferentialTestCase, UtilTestCase,
-         TestMergeFunctionMetadata, DeprecationTests, InitGroupsTests,
-         GetPasswordTest,
+    del (SwitchUIDTests, SearchUpwardsTests, RunAsEffectiveUserTests,
+         OrderedDictTests, IntervalDifferentialTests, UtilTests,
+         MergeFunctionMetadataTests, DeprecationTests, InitGroupsTests,
+         GetPasswordTests,
          )
