@@ -41,7 +41,7 @@ def getDivisionFailure(*args, **kwargs):
     return f
 
 
-class FailureTestCase(SynchronousTestCase):
+class FailureTests(SynchronousTestCase):
     """
     Tests for L{failure.Failure}.
     """
@@ -217,6 +217,9 @@ class FailureTestCase(SynchronousTestCase):
         """
         if captureVars:
             exampleLocalVar = 'xyz'
+            # Silence the linter as this variable is checked via
+            # the traceback.
+            exampleLocalVar
 
         f = getDivisionFailure(captureVars=captureVars)
         out = NativeStringIO()
@@ -272,6 +275,9 @@ class FailureTestCase(SynchronousTestCase):
         """
         if captureVars:
             exampleLocalVar = 'abcde'
+            # Silence the linter as this variable is checked via
+            # the traceback.
+            exampleLocalVar
 
         f = getDivisionFailure()
         out = NativeStringIO()
@@ -319,6 +325,9 @@ class FailureTestCase(SynchronousTestCase):
         """
         if captureVars:
             exampleLocalVar = 'xyzzy'
+            # Silence the linter as this variable is checked via
+            # the traceback.
+            exampleLocalVar
 
         f = getDivisionFailure(captureVars=captureVars)
         out = NativeStringIO()
@@ -410,7 +419,7 @@ class FailureTestCase(SynchronousTestCase):
             detail='noisia')
 
 
-    def testExplictPass(self):
+    def test_ExplictPass(self):
         e = RuntimeError()
         f = failure.Failure(e)
         f.trap(RuntimeError)
@@ -428,23 +437,12 @@ class FailureTestCase(SynchronousTestCase):
                 "f.raiseException() didn't raise ZeroDivisionError!?")
 
 
-    def testRaiseExceptionWithTB(self):
+    def test_RaiseExceptionWithTB(self):
         f = getDivisionFailure()
         innerline = self._getInnermostFrameLine(f)
         self.assertEqual(innerline, '1/0')
 
 
-    def testLackOfTB(self):
-        f = getDivisionFailure()
-        f.cleanFailure()
-        innerline = self._getInnermostFrameLine(f)
-        self.assertEqual(innerline, '1/0')
-
-    testLackOfTB.todo = "the traceback is not preserved, exarkun said he'll try to fix this! god knows how"
-    if _PY3:
-        del testLackOfTB # fix in ticket #6008
-
-
     def test_stringExceptionConstruction(self):
         """
         Constructing a C{Failure} with a string as its exception value raises
@@ -454,7 +452,7 @@ class FailureTestCase(SynchronousTestCase):
         self.assertIn("Strings are not supported by Failure", str(exc))
 
 
-    def testConstructionFails(self):
+    def test_ConstructionFails(self):
         """
         Creating a Failure with no arguments causes it to try to discover the
         current interpreter exception state.  If no such state exists, creating
@@ -740,7 +738,7 @@ class FindFailureTests(SynchronousTestCase):
 
 
 
-class TestFormattableTraceback(SynchronousTestCase):
+class FormattableTracebackTests(SynchronousTestCase):
     """
     Whitebox tests that show that L{failure._Traceback} constructs objects that
     can be used by L{traceback.extract_tb}.
@@ -778,7 +776,7 @@ class TestFormattableTraceback(SynchronousTestCase):
 
 
 
-class TestFrameAttributes(SynchronousTestCase):
+class FrameAttributesTests(SynchronousTestCase):
     """
     _Frame objects should possess some basic attributes that qualify them as
     fake python Frame objects.
@@ -797,7 +795,7 @@ class TestFrameAttributes(SynchronousTestCase):
 
 
 
-class TestDebugMode(SynchronousTestCase):
+class DebugModeTests(SynchronousTestCase):
     """
     Failure's debug mode should allow jumping into the debugger.
     """
@@ -944,13 +942,11 @@ class ExtendedGeneratorTests(SynchronousTestCase):
         self.assertEqual(newFailures[0].getTraceback(), f.getTraceback())
 
     if _PY3:
-        test_findFailureInGenerator.todo = (
+        # FIXME: https://twistedmatrix.com/trac/ticket/5949
+        test_findFailureInGenerator.skip = (
             "Python 3 support to be fixed in #5949")
-        test_failureConstructionFindsOriginalFailure.todo = (
+        test_failureConstructionFindsOriginalFailure.skip = (
             "Python 3 support to be fixed in #5949")
-        # Remove these two lines in #6008 (unittest todo support):
-        del test_findFailureInGenerator
-        del test_failureConstructionFindsOriginalFailure
 
 
     def test_ambiguousFailureInGenerator(self):
