@@ -29,11 +29,13 @@ from twisted.internet import defer
 from twisted.trial import util, unittest
 from twisted.trial.itrial import ITestCase
 from twisted.trial.reporter import _ExitWrapper, UncleanWarningsReporterWrapper
+from twisted.trial._asyncrunner import _ForceGarbageCollectionDecorator, _iterateTests
+from twisted.trial._synctest import _logObserver
 
 # These are imported so that they remain in the public API for t.trial.runner
 from twisted.trial.unittest import TestSuite
 
-from zope.interface import implements
+from zope.interface import implementer
 
 pyunit = __import__('unittest')
 
@@ -173,7 +175,7 @@ class LoggedSuite(TestSuite):
 
         @param result: A L{TestResult} object.
         """
-        observer = unittest._logObserver
+        observer = _logObserver
         observer._add()
         super(LoggedSuite, self).run(result)
         observer._remove()
@@ -195,7 +197,7 @@ class TrialSuite(TestSuite):
             newTests = []
             for test in tests:
                 test = unittest.decorate(
-                    test, unittest._ForceGarbageCollectionDecorator)
+                    test, _ForceGarbageCollectionDecorator)
                 newTests.append(test)
             tests = newTests
         suite = LoggedSuite(tests)
@@ -252,14 +254,13 @@ def isTestCase(obj):
 
 
 
+@implementer(ITestCase)
 class TestHolder(object):
     """
     Placeholder for a L{TestCase} inside a reporter. As far as a L{TestResult}
     is concerned, this looks exactly like a unit test.
     """
 
-    implements(ITestCase)
-
     failureException = None
 
     def __init__(self, description):
@@ -319,10 +320,8 @@ class ErrorHolder(TestHolder):
 
 
     def __repr__(self):
-        return "<ErrorHolder description=%r error=%s%s>" % (
-            # Format the exception type and arguments explicitly, as exception
-            # objects do not have nice looking string formats on Python 2.4.
-            self.description, self.error[0].__name__, self.error[1].args)
+        return "<ErrorHolder description=%r error=%r>" % (
+            self.description, self.error[1])
 
 
     def run(self, result):
@@ -424,8 +423,8 @@ class TestLoader(object):
         if not hasattr(module, '__doctests__'):
             return suite
         docSuite = self.suiteFactory()
-        for doctest in module.__doctests__:
-            docSuite.addTest(self.loadDoctests(doctest))
+        for docTest in module.__doctests__:
+            docSuite.addTest(self.loadDoctests(docTest))
         return self.suiteFactory([suite, docSuite])
     loadTestsFromModule = loadModule
 
@@ -471,7 +470,7 @@ class TestLoader(object):
         Tests are only loaded from modules whose name begins with 'test_'
         (or whatever C{modulePrefix} is set to).
 
-        @param package: a types.ModuleType object (or reasonable facsimilie
+        @param package: a types.ModuleType object (or reasonable facsimile
         obtained by importing) which may contain tests.
 
         @param recurse: A boolean.  If True, inspect modules within packages
@@ -742,7 +741,7 @@ class TrialRunner(object):
         suite = TrialSuite([test], forceGarbageCollection)
         startTime = time.time()
         if self.mode == self.DRY_RUN:
-            for single in unittest._iterateTests(suite):
+            for single in _iterateTests(suite):
                 result.startTest(single)
                 result.addSuccess(single)
                 result.stopTest(single)
