@@ -3,13 +3,14 @@
 # See LICENSE for details.
 
 """
-Twisted Web: a L{web server<twisted.web.server>} (including an
-L{HTTP implementation<twisted.web.http>} and a
-L{resource model<twisted.web.resource>}) and
+Twisted Web: HTTP clients and servers, plus tools for implementing them.
+
+Contains a L{web server<twisted.web.server>} (including an
+L{HTTP implementation<twisted.web.http>}, a
+L{resource model<twisted.web.resource>}), and
 a L{web client<twisted.web.client>}.
 """
 
 from twisted.web._version import version
 
 __version__ = version.short()
-
